terraform {
  required_providers {
    azurerm = {
      source  = "hashicorp/azurerm"
      version = "=2.46.0"
    }
  }
}

provider "azurerm" {
    subscription_id = ""
    client_id = ""
    client_secret = ""
    tenant_id = ""
    features {}
}

# Create a resource group if it doesn't exist
resource "azurerm_resource_group" "proxylogon_resources" {
    name     = "${var.var_prefix}resources"
    location = var.var_location
}

# Create virtual network
resource "azurerm_virtual_network" "proxylogon_network" {
    name                = "${var.var_prefix}network"
    address_space       = ["10.0.0.0/16", "172.16.0.0/16"]
    location            = var.var_location

    resource_group_name = azurerm_resource_group.proxylogon_resources.name
}

# Create subnet
resource "azurerm_subnet" "proxylogon_subnet" {
    name                 = "${var.var_prefix}${var.var_subnets[count.index].name}subnet"
    resource_group_name  = azurerm_resource_group.proxylogon_resources.name

    count = length(var.var_subnets)

    virtual_network_name = azurerm_virtual_network.proxylogon_network.name
    address_prefixes       = [var.var_subnets[count.index].ip_range]
}

# Create public IPs
resource "azurerm_public_ip" "proxylogon_public_ips" {
    name                         = "${var.var_prefix}${var.var_vms[count.index].name}publicIPs"
    location                     = var.var_location
    resource_group_name          = azurerm_resource_group.proxylogon_resources.name

    count = length(var.var_vms)

    allocation_method            = "Dynamic"
    domain_name_label = lower("${var.var_prefix}-${var.var_vms[count.index].name}")
}

# Create Network Security Group and rule
resource "azurerm_network_security_group" "proxylogon_security_group" {
    name                = "${var.var_prefix}securitygroup"
    location            = var.var_location
    resource_group_name = azurerm_resource_group.proxylogon_resources.name

    security_rule {
        name                       = "SSH"
        priority                   = 1001
        direction                  = "Inbound"
        access                     = "Allow"
        protocol                   = "Tcp"
        source_port_range          = "*"
        destination_port_range     = "22"
        source_address_prefix      = "*"
        destination_address_prefix = "*"
    }

    security_rule {
        name                       = "HTTP"
        priority                   = 1002
        direction                  = "Inbound"
        access                     = "Allow"
        protocol                   = "Tcp"
        source_port_range          = "*"
        destination_port_range     = "80"
        source_address_prefix      = "*"
        destination_address_prefix = "*"
    }

    security_rule {
        name                       = "HTTPS"
        priority                   = 1003
        direction                  = "Inbound"
        access                     = "Allow"
        protocol                   = "Tcp"
        source_port_range          = "443"
        destination_port_range     = "*"
        source_address_prefix      = "*"
        destination_address_prefix = "*"
    }

    security_rule {
        name                       = "RDP"
        priority                   = 1004
        direction                  = "Inbound"
        access                     = "Allow"
        protocol                   = "Tcp"
        source_port_range          = "3389"
        destination_port_range     = "*"
        source_address_prefix      = "*"
        destination_address_prefix = "*"
    }
  
    security_rule {
        name                       = "Any"
        priority                   = 1005
        direction                  = "Inbound"
        access                     = "Allow"
        protocol                   = "Tcp"
        source_port_range          = "*"
        destination_port_range     = "*"
        source_address_prefix      = "*"
        destination_address_prefix = "*"
    }
}
# Create network interface
resource "azurerm_network_interface" "proxylogon_network_interface" {
    name                      = "${var.var_prefix}${var.var_vms[count.index].name}nic"
    location                  = var.var_location
    resource_group_name       = azurerm_resource_group.proxylogon_resources.name

    count = length(var.var_vms)

    ip_configuration {
        name                          = "${var.var_prefix}${var.var_vms[count.index].name}NIC"
        subnet_id                     = azurerm_subnet.proxylogon_subnet[var.var_vms[count.index].subnet].id
        private_ip_address_allocation = "Static"
        private_ip_address            = var.var_vms[count.index].ip
        public_ip_address_id          = azurerm_public_ip.proxylogon_public_ips[count.index].id
    }
}

# Connect the security group to the network interface
resource "azurerm_network_interface_security_group_association" "proxylogon_group_association" {
    count = length(var.var_vms)
    
    network_interface_id      = azurerm_network_interface.proxylogon_network_interface[count.index].id
    network_security_group_id = azurerm_network_security_group.proxylogon_security_group.id
}

# Create virtual machine
resource "azurerm_windows_virtual_machine" "proxylogon_vms" {
    name                  = "${var.var_prefix}${var.var_vms[count.index].name}"
    location              = var.var_location
    resource_group_name   = azurerm_resource_group.proxylogon_resources.name

    count = 2

    network_interface_ids = [azurerm_network_interface.proxylogon_network_interface[count.index].id]
    size                  = var.var_vms[count.index].size

    os_disk {
        name              = "${var.var_prefix}${var.var_vms[count.index].name}disk"
        caching           = "ReadWrite"
        storage_account_type = "Standard_LRS"
    }

    source_image_reference {
        publisher = var.var_vms[count.index].publisher
        offer = var.var_vms[count.index].offer
        sku = var.var_vms[count.index].sku
        version = var.var_vms[count.index].version
    }

    computer_name  = "${var.var_prefix}${var.var_vms[count.index].name}"
    # disable_password_authentication = false
    admin_username = "mickey"
    admin_password = "3Y_1nt3rnsh1p!"
}

resource "azurerm_virtual_machine_extension" "enablewinrm" {
  count                 = 2
  name                  = "enablewinrm"
  virtual_machine_id    = azurerm_windows_virtual_machine.proxylogon_vms[count.index].id
  publisher            = "Microsoft.Compute" ## az vm extension image list --location eastus Do not use Microsoft.Azure.Extensions here
  type                 = "CustomScriptExtension" ## az vm extension image list --location eastus Only use CustomScriptExtension here
  type_handler_version = "1.9" ## az vm extension image list --location eastus
  auto_upgrade_minor_version = true
  settings = <<SETTINGS
    {
      "fileUris": ["https://raw.githubusercontent.com/ansible/ansible/devel/examples/scripts/ConfigureRemotingForAnsible.ps1"],
      "commandToExecute": "powershell -ExecutionPolicy Unrestricted -File ConfigureRemotingForAnsible.ps1"
    }
SETTINGS
}